// Wi-Canne MHK V2.3                          
// Refonte de la WiCanne_Ariane développé lors du FRABIKARIUM  ARIANE
// Hardware : Lolin D32 (ESP32) avec Bluetooth 4.2 intégré + L293D pour piloter le vibreur
//            + 4 capteurs laser VL53L1X  +  vibreur Gotronic VM1201  + Buzzer piézo + inter "changement de mode"
//            + inter "mise en route" + inter "sans init." + résist. 1 kOhm  + 2x résist. 10 kOhm  + SD card
// contrôle et modif paramètre sur appli Android "Wicanne 2.3"
// YLC 06/12/2019
/*******************************************************************
  Config :    (angles donnés par rapport à l'horizontale)
  laser 0 = mesure verticale de la distance au sol (90° vers le bas)
  laser 1 = mesure oblique (70° vers l'avant bas)
  laser 2 = mesure horizontale (0° vers l'avant)
  laser 3 = mesure oblique 45° vers l'avant haut
  NB : L'initialisation du système, si activée, doit être faite sur sol plat sans obstacle à moins de 3m.
***********************************************************************/
#include <SPI.h>
#include <mySD.h>
#include <Wire.h>
#include <VL53L1X.h>          
#include "notes.h"            // Notes pour jingle de fin d'initialisation  (CF onglet notes.h)
#include "BluetoothSerial.h"
BluetoothSerial SerialBT;
#include "EasyBuzzer.h"       // Attention : utilise le channel ledcpwm 0

#define TRACES 1              // 1 = trace sur port série ordi, 0 pour la désactiver
#define NB_CAPTEURS 4         // Nombre de capteurs IR
#define NB_HIST 3             // Nombre de mesures laser stockées dans l'historique
#define NOTE_ALERT_SOL1 800   // Note 1 d'alarme pour variation hauteur du sol
#define NOTE_ALERT_SOL2 1200  // Note 2 d'alarme pour variation hauteur du sol
#define NOTE_ALERT_SOL3 400   // Note 3 d'alarme pour variation hauteur du sol
#define NOTE_ALERT_SOL4 800   // Note 4 d'alarme pour variation hauteur du sol
#define NOTE_ALERT_TET1 2500  // Note 1 d'alerte d'un obstacle haut sur laser oblique haut n°3
#define NOTE_ALERT_TET2 2400  // Note 2 d'alerte d'un obstacle haut sur laser oblique haut n°3
#define NOTE_INIT_KO    2200  // Note d'alarme pour mauvaise initialisation

// brochage Lolin D32   (ne pas utiliser la pin 12 qui sert au bootloader)
// pin 13 en sortie pour le Buzzer  (cf : § setup)
const int swinit  = 15;       // pin 15 : swith pour activer ou non l'initialisation (interruption 0)
const int swmode  = 16;       // pin 16 : swith pour changer de mode (interruption 1) 
// pin 21 et 22 pour I2C  (cf : § setup)
const int vibreur = 25;       // pin 25 : commande vitesse vibreur    (pin 1 du L293D)
const int vibinp1 = 33;       // pin 33 : commande 1 sens de rotation (pin 2 du L293D)
const int vibinp2 = 32;       // pin 32 : commande 2 sens de rotation (pin 7 du L293D)
const int sdpin = 5;          // pin 5  : SD card (pin CS)
const int xshut[NB_CAPTEURS] = {17, 14, 27, 26}; // Pins xshut des capteurs laser VL53L1X pour nommage


// variable SD card
bool sdabs = true;           // indicateur d'absence de carte SD
bool sdko = true;            // indicateur erreur trt carte SD
File fichier;                // objet fichier 

// mode de détection 0=éloigné (ex.dans la rue)  1= proche (ex.dans le métro)  
int mode = 0;                 // indicateur du mode sélectionné (éloigné par défaut)
bool imode = false;           // indicateur de changement du mode

// Compteurs de boucles / indices de tableaux
int i = 0;
int j = 0;
int k = 0;
int ihist = 0;                           // indice pour stockage dans historique

int taille = 170;                        // Taille utilisateur en cm par défaut 
bool initok = false;                     // indicateur de bonne initialisation
bool indinit = true;                     // indicateur si initialisation ou non au démarrage
bool iinit = false;                      // indicateur de changement de l'initialisation

// bluetooth
String cmd = "";                         // chaine reçue par le Bluetooth 
int valmodif = 0;                        // valeur numérique de la valeur de la modif recue par bluetooth
boolean mess_complet = false;            // indicateur de reception d'un message bluetooth complet

// capteurs
const int addr[NB_CAPTEURS]  = {0x25, 0x30, 0x35, 0x40}; // adresses I2C pour nommage des capteurs laser VL53L1X
const double delay_init      = 50.0;     // délai de temporisation de l'initialisation des capteurs
const double timeout_capteur = 100.0;    // durée du timeout capteur laser
const double delai_mesure    = 10.0;     // délai de temporisation des mesures laser 
double mesure[NB_CAPTEURS];              // Mesures fournies par les lasers
double hist_data[NB_CAPTEURS][NB_HIST];  // stockage des mesures lasers (pour moyennage)
double dist_verti = 0;                   // hauteur au dessus du sol en cm à l'initialisation  du laser vertical n°0
double dist_bas = 0;                     // distance oblique du sol en cm à l'initialisation du laser oblique bas n°1
double var_bas = 0;                      // variation mesure du capteur oblique n°2 par rapport à la valeur sur sol plat
double min_haut = 0;                     // distance mini pour détection d'un obstacle haut en cm
double dist_haut = 0;                    // distance oblique vers le haut en cm du laser oblique haut n°3
double dist_horiz = 0;                   // distance mesurée par le laser horizontal n° 2
double mes_moy[NB_CAPTEURS];             // moyenne des dernières mesures des capteur en cm
double var_vertic = 0;                   // variation de la hauteur verticale au sol
    
int ialert[4] = {0, 0, 0, 0};            // indicateurs d'alerte 
bool muet = false;                       // indicateur de fonctionnement sans alarme sonore

// variables vibreur  (Setting PWM properties)
const int freq = 3000;
const int pwmChannel = 2;
const int resolution = 8;
int dutyCycle = 150;
int valvib;                                    // force de vibration à appliquer au vibreur
int paldis[4] = {30, 80, 150, 250};            // 4 paliers des distances horizontales pour mode proche (en cm)
int palvar[3] = {20, 40, 80};                  // 3 paliers des variations de mesures obliques au sol (en cm)
int forvib[4] = {255, 200, 160, 80};           // forces de vibrations correspondant à 4 paliers de distances
int dismaxh = 0;                               // variable de calcul distance max horizontale en mode proche

int melody[] = {                         // notes du jingle de fin d'initialisation (Mario)
  NOTE_E7, NOTE_E7, 0, NOTE_E7, 0, NOTE_C7, NOTE_E7, 0, NOTE_G7, 0, 0, 0, NOTE_G6, 0, 0, 0
};
int noteDurations[] = {                  // note durations: 4 = quarter note, 8 = eighth note, etc.:
  12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12, 12,
};

VL53L1X sensor[NB_CAPTEURS];       // déclaration capteurs laser VL53L1X

//=========================================================
void musique_init() {         // musique de fin d'initialisation
  // iterate over the notes of the melody:
  for (int thisNote = 0; thisNote < 16; thisNote++) {
    // to calculate the note duration, take one second divided by the note type.
    // e.g. quarter note = 1000 / 4, eighth note = 1000/8, etc.
    int noteDuration = 1000 / noteDurations[thisNote];
    EasyBuzzer.singleBeep(melody[thisNote], noteDuration);
    // to distinguish the notes, set a minimum time between them.
    // the note's duration + 30% seems to work well:
    int pauseBetweenNotes = noteDuration * 1.30;
    delay(pauseBetweenNotes);
  }
  EasyBuzzer.stopBeep();
}
//- - - - - - - - - - - - - - - - - - - - - - - - - - - -
void decalage_historique() {             // décalage vers la gauche des mesures dans l'historique...
  for (int k = 0; k < NB_CAPTEURS; k++) { // ...pour chaque laser
    for (int j = 1; j < NB_HIST; j++) {
      hist_data[k][j - 1] = hist_data[k][j];
    }
  }
}
//- - - - - - - - - - - - - - - - - - - - - - - - - - - -
void recuperer_data() {
  decalage_historique();
  for (k = 0; k < NB_CAPTEURS; k++) {
    mesure[k] = sensor[k].read();        // lecture des mesures par les capteurs laser en mm
    if (sensor[k].timeoutOccurred()) {
#if TRACES
      Serial.print("Timeout sensor : ");
      Serial.println(k);
#endif
      mesure[k] = 8888;                 // la mesure est non valide
    }
    else {
      if ((mesure[k] < 4000) && (mesure[k] > 100)) {    // mesure valide dans la plage 10 cm à 4 m.
        hist_data[k][ihist]  = mesure[k] / 10;          // on la stocke dans l'historique en cm
      }
      else {
        mesure[k] = 8888;               // la mesure est non valide
      }
    }
  }
  delay(delai_mesure);
  if (mesure[k] != 8888) {          // si donnée lue est valide et donc a été stockée
    ihist = ihist + 1;             // on change de poste dans l'historique des mesures
    if (ihist >= NB_HIST) {        // ou si historique plein...
      ihist = NB_HIST - 1;         // ...stockage dans le dernier poste
    }
  }
}
//- - - - - - - - - - - - - - - - - - - - - - - - - - - -
double recuperer_moyenne(int k) {
  double moy;
  int nmes = 0;

  // Moyenne des n dernieres mesures du capteur k
  moy = 0.0;
  for (j = 0; j < NB_HIST; j++) {
    if (hist_data[k][j] < 400) {
      moy = moy + hist_data[k][j];
      nmes = nmes + 1;
    }
  }
  moy = moy / double(nmes);
  return moy;
}
//- - - - - - - - - - - - - - - - - - - - - - - - - - - -
void initialisation() {
  ihist = 0;
  // On effectue les premières mesures des lasers pour remplir l'historique
  // on répète la boucle 3 fois pour être sûr que les lasers sont bien initialisés
  for (i = 0; i < 3; i++) {
    for (j = 0; j < NB_HIST; j++) { // on remplit l'historique lors de l'initialisation
      recuperer_data();
    }
  }

  // Si pas d'initialisation activé on utilise les valeurs stockées dans la carte SD... 
  // .....sinon on stocke dans la carte SD les valeurs calculées.
  if (indinit) {                  // initialisation demandée
    // calcul hauteur en cm du laser 0 au dessus du sol (varie selon la taille de l'utilisateur)
    dist_verti = recuperer_moyenne(0);
    // Calcul de la distance mini en cm pour détection d'un obstacle haut (capteur incliné à 45°)
    min_haut = ((taille) - dist_verti) / cos(radians(45));
    // calcul de la distance oblique sur sol plat en cm du laser n°1
    dist_bas = recuperer_moyenne(1);
    dist_horiz = recuperer_moyenne(2);
    dist_haut = recuperer_moyenne(3);
    dismaxh = paldis[3];          // distance max horizontale pour alerte (en mode proche)

    // contrôle de la bonne initialisation des mesures des capteurs
    // le laser n° 1 devrait mesurer entre 2m70 et 3m20 selon la + ou - bonne position de l'appareil)
    // les lasers n° 2 et 3 ne devraient pas avoir de mesure inférieure à 3m
    //  if (dist_bas < 270 || dist_bas > 320 || dist_horiz < 300 || dist_haut < 300)  {  // A DECOMMENTER EN FIN TESTS EN INTERIEUR
    if (dist_bas < 100 || dist_bas > 320 || dist_horiz <= 100 || dist_haut <= 100)  {    // VALEURS POUR TESTS EN INTERIEUR
      alarm(NOTE_INIT_KO, NOTE_INIT_KO); // ...on déclenche l'alarme de mauvaise initialisation
      initok = false;                    // mauvaise initialisation due à des obstacles
    } else {
        initok = true;                   // bonne initialisation
        if (!sdabs) {                    // si carte sd présente
          suppresparam2();
          enrparam2();                   // stocker les valeurs calculées dans carte sd          
        }  
      }   
  } else {                               // pas d'initialisation demandée
      if (sdabs) initok = false;         // mauvaise ré-initialisation due à absence de carte SD
      else {
        lecparam2();
        if (!sdko) {                     // si lecture ok
          parsecmd2();                   // restaurer les valeurs lues dans variables de service....  
          initok = true;                 // bonne initialisation 
        } else initok = false;           // mauvaise ré-initialisation due à erreur lecture carte SD
      }      
    }

    if (initok) {
      if (!muet && indinit)   musique_init();     // jouer la musique de fin d'initialisation
      vibration(250, 500);
    }
  }
//- - - - - - - - - - - - - - - - - - - - - - - - - - - -
// activation de l'alarme sonore
void alarm(int note1, int note2) {
  if (!muet) {
    EasyBuzzer.singleBeep(note1, 100);
    delay(50);
    if (note2 != note1) {
      EasyBuzzer.singleBeep(note2, 100);
      delay(100);
    }
    EasyBuzzer.stopBeep();
  }
}
//- - - - - - - - - - - - - - - - - - - - - - - - - - - -
int calforhor(int dist) {          // calcul force de vibration en fonction inverse de la distance horizontale...
  int force = 0;                   // ... selon le palier de mesure avec la force correspondante à ce palier
  if (dist <= paldis[0])       force = forvib[0];
  else if (dist <= paldis[1])  force = forvib[1];
  else if (dist <= paldis[2])  force = forvib[2];
  else                         force = forvib[3];
  return force;
}
//- - - - - - - - - - - - - - - - - - - - - - - - - - - -
int calforobl(int dist) {          // calcul force de vibration en fonction inverse de la distance oblique...
  int force = 0;                   // ... selon le palier de mesure avec la force correspondante à ce palier
  if (dist < palvar[0])       force = forvib[3];
  else if (dist < palvar[1])  force = forvib[2];
  else if (dist < palvar[2])  force = forvib[1];
  else                        force = forvib[3];
  return force;
}
//- - - - - - - - - - - - - - - - - - - - - - - - - - - -
void vibration(int niveau, int duree) {
  digitalWrite(vibinp1, HIGH);       // activer vibreur     
  digitalWrite(vibinp2, LOW);
  if (niveau > 255)  niveau = 255;   // plafonner force PWM de vibration 
  ledcWrite(pwmChannel, niveau);     // vibrer au niveau demandé 
  delay(duree);
  digitalWrite(vibinp1, LOW);        // arreter vibreur     
  digitalWrite(vibinp2, LOW);      
}
//- - - - - - - - - - - - - - - - - - - - - - - - - - - -
void scani2c() {                   // routine pour vérif des adresses sur bus I2C
  byte error, address;
  int nDevices;
  Serial.println("Scanning I2C...");
  nDevices = 0;
  for (address = 8; address < 127; address++ )  {
    Wire.beginTransmission(address);    //On utilise la valeur de retour du Wire.endTransmission()...
    error = Wire.endTransmission();     // ...pour voir si un periphérique reconnaît cette adresse.
    if (error == 0) {
      Serial.print("I2C device found at address 0x");
      if (address < 16)  Serial.print("0");
      Serial.println(address, HEX);
      nDevices++;
    }
    else if (error == 4)   {
      Serial.print("Unknown error at address 0x");
      if (address < 16)  Serial.print("0");
      Serial.println(address, HEX);
    }
  }
  if (nDevices == 0)  {
    Serial.println("No I2C devices found");
  }
}
//- - - - - - - - - - - - - - - - - - - - - - - - - - - -
void lect_bluetooth() {
  // lecture ordres recus par bluetooth
  while (SerialBT.available()) {       //  On traite les données reçues en entrée, même si partielles 
    char inchar = (char)SerialBT.read();
    switch (inchar) {  
      case '<' :                            // identifie un caractère de début de message 
        mess_complet = false;
        cmd = "";                           // on vide le champ de reconstruction du message
        break;
      case '>' :  
        mess_complet = true;                // identifie un caractère de fin de message 
        cmd += '\0';                        // mettre caractere fin de chaine
        break;
      default :
        cmd += inchar;                      // reconstruction du message par concaténation
    }     
  }
}
//- - - - - - - - - - - - - - - - - - - - - - - - - - - -
void modif_param()  {
  if (cmd.substring(0,2) == "Mp")   {       // test si modif des paramètres
    parsecmd1();                            // mise à jour des paramètres
    suppresparam1();                        // suppression fichier paramètres
    enrparam1();                            // stockage dans fichier paramètres
  } 
  initok = false;      // la modif d'un paramètre relance la proc d'initialisation
  while (!initok) {
    initialisation();  // appel initialisation tant que pas OK
  }
}
//- - - - - - - - - - - - - - - - - - - - - - - - - - - -
void parsecmd1() {  // parsing du string param1 ("Mp";taille;paldis0;paldis1;paldis2;paldis3;palvar0;palvar1;palvar2;)
   i = cmd.indexOf(';');                   // i = position du premier délimiteur (après Mp)
   j = cmd.indexOf(';',i+1);               // i = position du premier délimiteur (après Mp)
   taille = cmd.substring(i+1, j+1).toInt();       // modif taille utilisateur
   i = cmd.indexOf(';', j+1);
   paldis[0] = cmd.substring(j+1, i+1).toInt();    // modif palier 1 dist.horiz.
   j = cmd.indexOf(';', i+1);
   paldis[1] = cmd.substring(i+1, j+1).toInt();    // modif palier 2 dist.horiz.
   i = cmd.indexOf(';', j+1);
   paldis[2] = cmd.substring(i+1, j+1).toInt();    // modif palier 3 dist.horiz.
   j = cmd.indexOf(';', i+1);
   paldis[3] = cmd.substring(i+1, j+1).toInt();    // modif palier 4 dist.horiz.
   i = cmd.indexOf(';', j+1);
   palvar[0] = cmd.substring(i+1, j+1).toInt();    // modif palier 1 variation dist oblique sol
   j = cmd.indexOf(';', i+1);
   palvar[1] = cmd.substring(i+1, j+1).toInt();    // modif palier 2 variation dist oblique sol
   i = cmd.indexOf(';', j+1);
   palvar[2] = cmd.substring(i+1, j+1).toInt();    // modif palier 3 variation dist oblique sol
}
//- - - - - - - - - - - - - - - - - - - - - - - - - - - -
void parsecmd2() {        // parsing du string param2 ("Init";dist_verti;min_haut;dist_bas;)
   i = cmd.indexOf(';');                           // i = position du premier délimiteur (après Init)
   j = cmd.indexOf(';',i+1);                       // i = position du premier délimiteur (après Init)
   dist_verti = cmd.substring(i+1, j+1).toInt();   // modif dist. verticale vers le bas
   i = cmd.indexOf(';', j+1);
   min_haut = cmd.substring(j+1, i+1).toInt();     // modif dist. minimale à 45° vers le haut
   j = cmd.indexOf(';', i+1);
   dist_bas = cmd.substring(i+1, j+1).toInt();     // modif dist. normale à 70° vers le bas
}
//- - - - - - - - - - - - - - - - - - - - - - - - - - - -
void changemode() {
  imode = true; 
}
//- - - - - - - - - - - - - - - - - - - - - - - - - - - -
void changeinit() {
  iinit = true; 
}
//=========================================================

void setup() {
  #if TRACES
    Serial.begin(115200);     // vitesse du port de communication série pour debug
  #endif    
  SerialBT.begin("ESP32");
  cmd.reserve(64);           // réserver 64 octets pour réception message bluetooth

  EasyBuzzer.setPin(13);     // pin 13 : Pin buzzer
 
  // affectation interruption de changement de mode (proche ou éloigné)
  pinMode(swmode, INPUT);
  attachInterrupt(digitalPinToInterrupt(swmode), changemode, CHANGE);
  if (digitalRead(swmode) == LOW)  mode = 0;   // mode éloigné
  else mode = 1;                               // mode proche

  // affectation interruption d'activation ou non de l'initialisation 
  pinMode(swinit, INPUT);
  attachInterrupt(digitalPinToInterrupt(swinit), changeinit, CHANGE);
  if (digitalRead(swinit) == LOW)  indinit = false;   // sans initialisation 
  else indinit = true;                                // avec initialisation 

  // initialisation des capteurs laser sur bus I2C   
  #if TRACES
    Serial.println("Demarrage bus I2C");
  #endif
  Wire.begin(21, 22);         // I2C   pin 21 = SDA     pin 22 = SCL    Fréquence 100 kHz
  Wire.setClock(400000);      // fréquence bus I2C à 400 kHz 

  for (k = 0; k < NB_CAPTEURS; k++) {
    pinMode(xshut[k], OUTPUT);
    digitalWrite(xshut[k], LOW);      // Extinction des capteurs laser
    delay(delay_init);
    pinMode(xshut[k], INPUT);         // Ouverture capteurs
    if (!sensor[k].init()) {          // test si capteur inactif
      #if TRACES
        Serial.print("Capteur : ");
        Serial.print(k);
        Serial.println("  NON TROUVE.   STOP !");
      #endif
      while (1);                       // fige le programme
    }
    delay(delay_init);
    sensor[k].setAddress(addr[k]);     // nommage des capteurs
    sensor[k].setTimeout(timeout_capteur);       
    sensor[k].setDistanceMode(VL53L1X::Long);
    sensor[k].setMeasurementTimingBudget(50000);
    sensor[k].startContinuous(50);    // Démarrage capteurs laser
    delay(delay_init);
  }
  #if TRACES
    scani2c();                    // vérif du nommage des capteurs laser
  #endif  
  
  // Extinction du vibreur
  pinMode(vibreur, OUTPUT);
  pinMode(vibinp1, OUTPUT);
  pinMode(vibinp2, OUTPUT);
  ledcSetup(pwmChannel, freq, resolution); // configure LED PWM functionalitites
  ledcAttachPin(vibreur, pwmChannel);      // attach the channel to the GPIO to be controlled 
  digitalWrite(vibinp1, LOW);              // pin 1 low + pin 2 low = vibreur arrêté au départ
  digitalWrite(vibinp2, LOW);         

  // lecture des paramètres sur carte SD
  initsdcard(); 
  if (!sdabs) {                      // si carte bien présente
    lecparam1();
    if (!sdko) parsecmd1();          // stocker les paramètres lus dans variables de service....   
    else enrparam1();                // ...sinon créer fichier PARAM1 avec valeurs par défaut
  }  

  initok = false; 
  while (!initok) {
    initialisation();  // appel initialisation tant que pas OK
  }
}
//=========================================================

void loop() { 
  EasyBuzzer.update();                           // correction bug dans lib EasyBuzzer
    // test si changement de l'initialisation
  if (iinit == true) {
    if (digitalRead(swinit) == LOW)  indinit = false;   // sans initialisation 
    else indinit = true;                                // avec initialisation 
    iinit = false;
  }

  // test si changement de mode
  if (imode == true) {
    if (digitalRead(swmode) == LOW)  mode = 0;   // mode éloigné
    else mode = 1;                               // mode proche
    initok = false;
    while (!initok) {    // réinitialisation des capteurs au changement de mode
      initialisation();  // appel initialisation tant que pas OK
    }
    imode = false;
  }

  // Récuperation des mesures lasers
  recuperer_data();
  for (k = 0; k < NB_CAPTEURS; k++) {
    mes_moy[k] = recuperer_moyenne(k);
    ialert[k] = 0;
  }

  // trottoir ou marche détecté par le laser vertical n°0 (dans tous les modes)
  var_vertic = dist_verti - mes_moy[0];
  if (var_vertic > 10) {                     // si capteur vertical détecte une variation de + de 10 cm vers le haut ...
    alarm(NOTE_ALERT_SOL1, NOTE_ALERT_SOL2); // ...on déclenche l'alarme de présence d'une marche ou obstacle immédiat
    ialert[1] = 2;                           // indic d'alarme sonore
  }
  if (var_vertic < -10) {                    // si capteur vertical détecte une variation de + de 10 cm vers le bas....
    alarm(NOTE_ALERT_SOL4, NOTE_ALERT_SOL3); // ...on déclenche l'alarme de présence d'un trou ou d'une déclivité
    ialert[1] = 2;                           // indic d'alarme sonore
  }

  // Protection tête (test si un obstacle haut est en dessous de la taille de l'utilisateur)  (dans tous les modes)
  if ((mes_moy[3] > 10)                      // si mesure valide (plus de 10 cm)
      && (mes_moy[3] < min_haut))  {         // si mesure inférieure à distance de sécurité
    alarm(NOTE_ALERT_TET1, NOTE_ALERT_TET2);
    ialert[2] = 2;                           // indic d'alarme sonore
  }

  if (ialert[1] == 0 && ialert[2] == 0) {    // alerte sonore est prioritaire sur vibrations
    // Détection de variation de la planéité du sol en mode éloigné (mode 0)
    if (mode == 0) {
      var_bas = dist_bas - mes_moy[1];       // variation mesure du capteur oblique n°2 par rapport à la valeur sur sol plat
      if (abs(var_bas) >= 15) {              // si la variation >= 15 cm
        valvib = int(calforobl(abs(var_bas))); // ...calcul force vibration en fonction de la variation de distance mesurée...
        vibration(valvib, valvib * 2);         // alerte utilisateur par vibration  (durée proport. a la force)
        ialert[0] = 1;                         // indic de vibration
      }
    }
    
    // détection horizontale par le laser n°2 en mode proche (mode 1)
    if (mode == 1) {
      dist_horiz = mes_moy[2];
      if (dist_horiz < dismaxh) {             // si mesure dans la plage de détection...      
        valvib = calforhor(dist_horiz);       // ...calcul force vibration pour la distance mesurée...
        vibration(valvib, valvib);            // alerte utilisateur par vibration  (durée proport. a la force)
        ialert[3] = 1;                        // indic de vibration
      }
    }
  }

  // envoi des données par bluetooth si demande d'envoi reçue (mesures en CM)
    lect_bluetooth();
    if (mess_complet && cmd != "")  {
      SerialBT.flush();                   // vider le buffer bluetooth
      switch (cmd[1]) {                   // test le second caractère de la commande reçue
        case 'i' :                        // pour Vibrer
          vibration(250,500); 
          break;            
        case 'o' :                        // pour Sonner
          muet = false;
          alarm(600,800);
          break; 
        case 'u' :                        // pour Muet
          muet = true;
          break;   
        case '0' :                        // pour S0
          SerialBT.print("E0;");
          SerialBT.print(mode);
          SerialBT.print(";");
          SerialBT.print(int(mes_moy[0]));
          SerialBT.print(";");
          SerialBT.print(int(mes_moy[1]));
          SerialBT.print(";");
          SerialBT.print(int(mes_moy[2]));
          SerialBT.print(";");
          SerialBT.println(int(mes_moy[3]));
          SerialBT.print(";");
          SerialBT.println(int(indinit));
          SerialBT.print(";");
          break;
        case '1' :                       // pour S1          
          SerialBT.print("E1;");
          if (mode == 0) {
            SerialBT.print(int(dist_bas));
            SerialBT.print(";");
            SerialBT.print(int(var_bas));
            SerialBT.print(";");
          } else SerialBT.print(" ; ;"); 
          SerialBT.print(int(dist_verti));
          SerialBT.print(";");
          SerialBT.println(int(var_vertic));
          SerialBT.print(";");
          break; 
        case '2' :                       // pour S2      
          SerialBT.print("E2;");
          SerialBT.print(int(min_haut));
          SerialBT.print(";");
          SerialBT.print(int(mes_moy[3]));
          SerialBT.print(";");
          if (mode == 1)  SerialBT.print(int(dist_horiz));
          else SerialBT.print(" ");
          for (i = 0; i <4; i++) {
            SerialBT.print(";");
            SerialBT.print(ialert[i]); 
          }
          SerialBT.print(";");
          break;
        case 'a' :                       // pour Par (demande affichage paramètres)      
          SerialBT.print("P1;");
          SerialBT.println(taille);      // taille utilisateur
          for (i = 0; i <4; i++) {
            SerialBT.print(";");
            SerialBT.print(paldis[i]);   // paliers dist horiz de mesure en mode proche
          }
          for (i = 0; i <3; i++) {
            SerialBT.print(";");
            SerialBT.print(palvar[i]);   // paliers de dist oblique sol mesure en mode éloigné
          }
          SerialBT.print(";");
          break; 
       case 'p' :                       // pour Mp (modifier paramètres)    
          modif_param();  
          SerialBT.println("OK;");
          break;     
      }
       cmd = "";      
    } 

}
//**********************fin prog******************************/
